@@ -203,7 +203,9 @@ int wcmDeviceTypeKeys(InputInfoPtr pInfo)
 			/* fall through */
 
 		case 0x34D: /* MobileStudio Pro 13 */
-		case 0x34E: /* MobileStudio Pro 13 */
+		case 0x34E: /* MobileStudio Pro 16 */
+		case 0x398: /* MobileStudio Pro 13 */
+		case 0x399: /* MobileStudio Pro 16 */
 			TabletSetFeature(priv->common, WCM_LCD);
 			/* fall through */
 
@@ -355,6 +357,8 @@ int wcmDeviceTypeKeys(InputInfoPtr pInfo)
 		case 0x382:/* DTK-2451 */
 		case 0x37D:/* DTH-2452 */
 		case 0x37E:/* DTH-2452 Touch */
+		case 0x39A:/* MobileStudio Pro 13 Touch */
+		case 0x39B:/* MobileStudio Pro 16 Touch */
 			TabletSetFeature(priv->common, WCM_LCD);
 			break;
 	}
@@ -922,13 +926,15 @@ Bool wcmPreInitParseOptions(InputInfoPtr pInfo, Bool is_primary,
 	if (wcmParseSerials (pInfo) != 0)
 		goto error;
 
-	if (IsCursor(priv))
+	if (IsTablet(priv))
 	{
-		common->wcmCursorProxoutDist = xf86SetIntOption(pInfo->options, "CursorProx", 0);
-		if (common->wcmCursorProxoutDist < 0 ||
-				common->wcmCursorProxoutDist > common->wcmMaxDist)
-			xf86Msg(X_CONFIG, "%s: CursorProx invalid %d \n",
-				pInfo->name, common->wcmCursorProxoutDist);
+		const char *prop = IsCursor(priv) ? "CursorProx" : "StylusProx";
+		priv->wcmProxoutDist = xf86SetIntOption(pInfo->options, prop, 0);
+		if (priv->wcmProxoutDist < 0 ||
+				priv->wcmProxoutDist > common->wcmMaxDist)
+			xf86Msg(X_CONFIG, "%s: %s invalid %d \n",
+				pInfo->name, prop, priv->wcmProxoutDist);
+		priv->wcmSurfaceDist = -1;
 	}
 
 	priv->topX = xf86SetIntOption(pInfo->options, "TopX", 0);
@@ -1013,13 +1019,10 @@ Bool wcmPreInitParseOptions(InputInfoPtr pInfo, Bool is_primary,
 	if (TabletHasFeature(common, WCM_2FGT))
 	{
 		int gesture_is_on;
-
-		/* GestureDefault was off for all devices
-		 * except when multi-touch is supported */
-		common->wcmGestureDefault = 1;
+		Bool gesture_default = TabletHasFeature(priv->common, WCM_LCD) ? FALSE : TRUE;
 
 		gesture_is_on = xf86SetBoolOption(pInfo->options, "Gesture",
-					    common->wcmGestureDefault);
+					    gesture_default);
 
 		if (is_primary || IsTouch(priv))
 			common->wcmGesture = gesture_is_on;
@@ -1062,11 +1065,11 @@ error:
 	return FALSE;
 }
 
-/* The values were based on trail and error. */
-#define WCM_BAMBOO3_MAXX 4096.0
-#define WCM_BAMBOO3_ZOOM_DISTANCE 180.0
-#define WCM_BAMBOO3_SCROLL_DISTANCE 80.0
-#define WCM_BAMBOO3_SCROLL_SPREAD_DISTANCE 350.0
+/* The values were based on trial and error with a 3rd-gen Bamboo */
+#define WCM_DEFAULT_MM_XRES           (27.8 * 1000)
+#define WCM_DEFAULT_MM_YRES           (44.5 * 1000)
+#define WCM_ZOOM_DISTANCE_MM          6.5
+#define WCM_SCROLL_DISTANCE_MM        1.8
 
 /**
  * Parse post-init options for this device. Useful for overriding HW
@@ -1093,10 +1096,10 @@ Bool wcmPostInitParseOptions(InputInfoPtr pInfo, Bool is_primary,
 	/* 2FG touch device */
 	if (TabletHasFeature(common, WCM_2FGT) && IsTouch(priv))
 	{
-		int zoom_distance = common->wcmMaxTouchX *
-			(WCM_BAMBOO3_ZOOM_DISTANCE / WCM_BAMBOO3_MAXX);
-		int scroll_distance = common->wcmMaxTouchX *
-			(WCM_BAMBOO3_SCROLL_DISTANCE / WCM_BAMBOO3_MAXX);
+		int x_res = common->wcmTouchResolX ? common->wcmTouchResolX : WCM_DEFAULT_MM_XRES;
+		int y_res = common->wcmTouchResolY ? common->wcmTouchResolY : WCM_DEFAULT_MM_YRES;
+		int zoom_distance = WCM_ZOOM_DISTANCE_MM * x_res / 1000;
+		int scroll_distance = WCM_SCROLL_DISTANCE_MM * y_res / 1000;
 
 		common->wcmGestureParameters.wcmZoomDistance =
 			xf86SetIntOption(pInfo->options, "ZoomDistance",
@@ -1105,10 +1108,6 @@ Bool wcmPostInitParseOptions(InputInfoPtr pInfo, Bool is_primary,
 		common->wcmGestureParameters.wcmScrollDistance =
 			xf86SetIntOption(pInfo->options, "ScrollDistance",
 					 scroll_distance);
-
-		common->wcmGestureParameters.wcmMaxScrollFingerSpread =
-			common->wcmMaxTouchX *
-			(WCM_BAMBOO3_SCROLL_SPREAD_DISTANCE / WCM_BAMBOO3_MAXX);
 	}
 
 
