@@ -48,9 +48,6 @@
 #define TILT_MIN -64		/* Minimum reported tilt value */
 #define TILT_MAX 63		/* Maximum reported tilt value */
 
-#define MIN_PAD_RING 0		/* I4 absolute scroll ring min value */
-#define MAX_PAD_RING 71		/* I4 absolute scroll ring max value */
-
 /* I4 cursor tool has a rotation offset of 175 degrees */
 #define INTUOS4_CURSOR_ROTATION_OFFSET 175
 
@@ -96,6 +93,12 @@
 #define ABS_MT_SLOT 0x2f
 #endif
 
+/* 4.15 */
+
+#ifndef BTN_STYLUS3
+#define BTN_STYLUS3 0x149
+#endif
+
 /******************************************************************************
  * Forward Declarations
  *****************************************************************************/
@@ -172,6 +175,7 @@ struct _WacomModel
 #define ABSOLUTE_FLAG		0x00000100
 #define BAUD_19200_FLAG		0x00000400
 #define BUTTONS_ONLY_FLAG	0x00000800
+#define SCROLLMODE_FLAG		0x00001000
 
 #define IsCursor(priv) (DEVICE_ID((priv)->flags) == CURSOR_ID)
 #define IsStylus(priv) (DEVICE_ID((priv)->flags) == STYLUS_ID)
@@ -183,10 +187,10 @@ struct _WacomModel
 
 #define IsUSBDevice(common) ((common)->wcmDevCls == &gWacomUSBDevice)
 
-#define FILTER_PRESSURE_RES	2048	/* maximum points in pressure curve */
+#define FILTER_PRESSURE_RES	65536	/* maximum points in pressure curve */
 /* Tested result for setting the pressure threshold to a reasonable value */
-#define THRESHOLD_TOLERANCE (FILTER_PRESSURE_RES / 125)
-#define DEFAULT_THRESHOLD (FILTER_PRESSURE_RES / 75)
+#define THRESHOLD_TOLERANCE (0.008f)
+#define DEFAULT_THRESHOLD (0.013f)
 
 #define WCM_MAX_BUTTONS		32	/* maximum number of tablet buttons */
 #define WCM_MAX_X11BUTTON	127	/* maximum button number X11 can handle */
@@ -235,8 +239,8 @@ struct _WacomDeviceState
 };
 
 static const struct _WacomDeviceState OUTPROX_STATE = {
-  .abswheel = MAX_PAD_RING + 1,
-  .abswheel2 = MAX_PAD_RING + 1
+  .abswheel = INT_MAX,
+  .abswheel2 = INT_MAX
 };
 
 struct _WacomDeviceRec
@@ -285,14 +289,17 @@ struct _WacomDeviceRec
 	WacomCommonPtr common;  /* common info pointer */
 
 	/* state fields in device coordinates */
+	struct _WacomDeviceState wcmPanscrollState; /* panscroll state tracking */
 	struct _WacomDeviceState oldState; /* previous state information */
 	int oldCursorHwProx;	/* previous cursor hardware proximity */
 
-	/* JEJ - filters */
-	int pPressCurve[FILTER_PRESSURE_RES + 1]; /* pressure curve */
+	int maxCurve;		/* maximum pressure curve value */
+	int *pPressCurve;       /* pressure curve */
 	int nPressCtrl[4];      /* control points for curve */
 	int minPressure;	/* the minimum pressure a pen may hold */
 	int oldMinPressure;     /* to record the last minPressure before going out of proximity */
+	int wcmSurfaceDist;	/* Distance reported by hardware when tool at surface */
+	int wcmProxoutDist;     /* Distance from surface when proximity-out should be triggered */
 	unsigned int eventCnt;  /* count number of events while in proximity */
 	int maxRawPressure;     /* maximum 'raw' pressure seen until first button event */
 	WacomToolPtr tool;         /* The common tool-structure for this device */
@@ -347,7 +354,7 @@ struct _WacomDeviceClass
 {
 	Bool (*Detect)(InputInfoPtr pInfo); /* detect device */
 	Bool (*ParseOptions)(InputInfoPtr pInfo); /* parse class-specific options */
-	Bool (*Init)(InputInfoPtr pInfo, char* id, float *version);   /* initialize device */
+	Bool (*Init)(InputInfoPtr pInfo, char* id, size_t id_len, float *version);   /* initialize device */
 	int  (*ProbeKeys)(InputInfoPtr pInfo); /* set the bits for the keys supported */
 };
 
@@ -368,7 +375,6 @@ typedef struct {
 	int wcmZoomDistance;	       /* minimum distance for a zoom touch gesture */
 	int wcmScrollDistance;	       /* minimum motion before sending a scroll gesture */
 	int wcmScrollDirection;	       /* store the vertical or horizontal bit in use */
-	int wcmMaxScrollFingerSpread; /* maximum distance between fingers for scroll gesture */
 	int wcmGestureUsed;	       /* retain used gesture count within one in-prox event */
 	int wcmTapTime;	   	       /* minimum time between taps for a right click */
 } WacomGesturesParameters;
@@ -436,6 +442,8 @@ struct _WacomCommonRec
 
 	int wcmMaxStripX;            /* Maximum fingerstrip X */
 	int wcmMaxStripY;            /* Maximum fingerstrip Y */
+	int wcmMinRing;              /* Minimum touchring value */
+	int wcmMaxRing;              /* Maximum touchring value */
 
 	WacomDevicePtr wcmDevices;   /* list of devices sharing same port */
 	int wcmPktLength;            /* length of a packet */
@@ -451,17 +459,15 @@ struct _WacomCommonRec
 	int wcmTouch;	             /* disable/enable touch event */
 	int wcmTouchDefault;	     /* default to disable when not supported */
 	int wcmGesture;	     	     /* disable/enable touch gesture */
-	int wcmGestureDefault;       /* default touch gesture to disable when not supported */
 	int wcmGestureMode;	       /* data is in Gesture Mode? */
 	WacomDeviceState wcmGestureState[MAX_FINGERS]; /* inital state when in gesture mode */
 	WacomGesturesParameters wcmGestureParameters;
-	int wcmMaxCursorDist;	     /* Max mouse distance reported so far */
-	int wcmCursorProxoutDist;    /* Max mouse distance for proxy-out max/256 units */
-	int wcmCursorProxoutDistDefault; /* Default max mouse distance for proxy-out */
+	int wcmProxoutDistDefault;   /* Default value for wcmProxoutDist */
 	int wcmSuppress;        	 /* transmit position on delta > supress */
 	int wcmRawSample;	     /* Number of raw data used to filter an event */
 	int wcmPressureRecalibration; /* Determine if pressure recalibration of
 					 worn pens should be performed */
+	int wcmPanscrollThreshold;	/* distance pen must move to send a panscroll event */
 
 	int bufpos;                        /* position with buffer */
 	unsigned char buffer[BUFFER_SIZE]; /* data read from device */
